@@ -1,20 +1,22 @@
 from contextlib import closing
 from datetime import datetime
 from io import BytesIO
-from logging import info
+from logging import getLogger
 
 from dropbox import Dropbox
-from dropbox.files import CreateFolderError, DeleteError, FileMetadata, FolderMetadata, WriteMode
+from dropbox.files import CreateFolderError, DeleteError, FileMetadata, FolderMetadata, ListFolderContinueError, ListFolderError, WriteMode
 from dropbox.exceptions import ApiError
 from fs.base import FS
 from fs.enums import ResourceType
-from fs.errors import DirectoryExists, DirectoryExpected, DirectoryNotEmpty, FileExists, FileExpected, ResourceNotFound
+from fs.errors import DirectoryExists, DirectoryExpected, DirectoryNotEmpty, FileExists, FileExpected, FSError, ResourceNotFound
 from fs.info import Info
 from fs.mode import Mode
 from fs.path import dirname
 from fs.subfs import SubFS
 from fs.time import datetime_to_epoch
 
+_logger = getLogger(__name__)
+
 def _infoFromMetadata(metadata):
 	rawInfo = {
 		'basic': {
@@ -75,7 +77,7 @@ def _infoFromMetadata(metadata):
 		assert False, f'{metadata.name}, {metadata}, {type(metadata)}'
 	return Info(rawInfo)
 
-class DropboxFile(BytesIO):
+class _DropboxFile(BytesIO):
 	def __init__(self, dropbox, path, mode):
 		self.dropbox = dropbox
 		self.path = path
@@ -88,15 +90,12 @@ class DropboxFile(BytesIO):
 			with closing(response):
 				if (self._mode.reading or self._mode.appending) and not self._mode.truncate:
 					initialData = response.content
-					info(f'Read initial data: {initialData}')
 		except ApiError:
 			# if the file doesn't exist, we don't need to read it's initial state
 			pass
 		super().__init__(initialData)
-		info(f'Set initial data: {initialData}, mode={self._mode}, app={self._mode.appending}')
 		if self._mode.appending and initialData is not None:
 			# seek to the end
-			info('Seeking to the end')
 			self.seek(len(initialData))
 		self._closed = False
 
@@ -138,7 +137,6 @@ class DropboxFile(BytesIO):
 		return self._mode.to_platform_bin()
 
 	def close(self):
-		info(f'close: {self.getvalue()}')
 		if not self._mode.writing and not self._mode.appending:
 			self._closed = True
 			return
@@ -146,7 +144,6 @@ class DropboxFile(BytesIO):
 			writeMode = WriteMode('add')
 		else:
 			writeMode = WriteMode('update', self.rev)
-		info(f'{writeMode=}')
 		metadata = self.dropbox.files_upload(self.getvalue(), self.path, mode=writeMode, autorename=False, client_modified=datetime.utcnow(), mute=False) # pylint: disable=unused-variable
 		# Make sure that we can't call this again
 		self.path = None
@@ -176,6 +173,7 @@ class DropboxFS(FS):
 		return 'DropboxFS()'
 
 	def getinfo(self, path, namespaces=None):
+		_logger.info(f'getinfo({path}, {namespaces})')
 		path = self.validatepath(path)
 		if path == '/':
 			return Info({'basic': {'name': '', 'is_dir': True}})
@@ -186,7 +184,8 @@ class DropboxFS(FS):
 				raise ResourceNotFound(path=path) from e
 			return _infoFromMetadata(metadata)
 
-	def setinfo(self, path, info): # pylint: disable=redefined-outer-name
+	def setinfo(self, path, info):
+		_logger.info(f'setinfo({path}, {info})')
 		# dropbox doesn't support changing any of the metadata values
 		path = self.validatepath(path)
 		with self._lock:
@@ -194,19 +193,24 @@ class DropboxFS(FS):
 				raise ResourceNotFound(path)
 
 	def listdir(self, path):
+		_logger.info(f'listdir({path})')
 		path = self.validatepath(path)
 		with self._lock:
 			return [x.name for x in self.scandir(path)]
 
 	def makedir(self, path, permissions=None, recreate=False):
+		_logger.info(f'makedir({path}, {permissions}, {recreate})')
 		path = self.validatepath(path)
 		with self._lock:
-			if self.exists(path):
-				if self.isdir(path):
+			try:
+				info_ = self.getinfo(path)
+				if info_.is_dir:
 					if recreate is False:
 						raise DirectoryExists(path=path)
 					return SubFS(self, path)
 				raise DirectoryExists(path)
+			except ResourceNotFound:
+				pass
 
 			# check that the parent directory exists
 			parentDir = dirname(path)
@@ -223,6 +227,7 @@ class DropboxFS(FS):
 			return SubFS(self, path)
 
 	def openbin(self, path, mode='r', buffering=-1, **options):
+		_logger.info(f'openbin({path}, {mode}, {buffering}, {options})')
 		if 't' in mode:
 			raise ValueError('Text mode is not allowed in openbin')
 		path = self.validatepath(path)
@@ -244,14 +249,15 @@ class DropboxFS(FS):
 				parentDir = dirname(path)
 				# throws ResourceNotFound if the parent dir doesn't exist
 				_ = self.getinfo(parentDir)
-			return DropboxFile(self.dropbox, path, mode)
+			return _DropboxFile(self.dropbox, path, mode)
 
 	def remove(self, path):
+		_logger.info(f'remove({path})')
 		path = self.validatepath(path)
 		with self._lock:
-			if self.exists(path) is False:
-				raise ResourceNotFound(path=path)
-			if self.isdir(path) is True:
+			# correctly throws ResourceNotFound if path doesn't exist
+			info_ = self.getinfo(path)
+			if info_.is_dir:
 				raise FileExpected(path=path)
 			try:
 				self.dropbox.files_delete_v2(path)
@@ -259,13 +265,14 @@ class DropboxFS(FS):
 				raise FileExpected(path=path) from e
 
 	def removedir(self, path):
+		_logger.info(f'removedir({path})')
 		path = self.validatepath(path)
 		with self._lock:
-			if self.exists(path) is False:
-				raise ResourceNotFound(path=path)
-			if self.isdir(path) is False:
+			# correctly throws ResourceNotFound if path doesn't exist
+			info_ = self.getinfo(path)
+			if info_.is_dir is False:
 				raise DirectoryExpected(path=path)
-			if len(self.listdir(path)) > 0:
+			if next(self.scandir(path), None) is not None:
 				raise DirectoryNotEmpty(path=path)
 			try:
 				self.dropbox.files_delete_v2(path)
@@ -275,20 +282,27 @@ class DropboxFS(FS):
 
 	# non-essential method - for speeding up walk
 	def scandir(self, path, namespaces=None, page=None):
+		_logger.info(f'scandir({path}, {namespaces}, {page})')
 		path = self.validatepath(path)
 		with self._lock:
-			if self.exists(path) is False:
-				raise ResourceNotFound(path=path)
-			if self.isdir(path) is False:
+			# correctly throws ResourceNotFound if path doesn't exist
+			info_ = self.getinfo(path)
+			if info_.is_dir is False:
 				raise DirectoryExpected(path=path)
 
 			# get all the avaliable metadata since it's cheap
 			# TODO - this call has a recursive flag so we can either use that and cache OR override walk
-			result = self.dropbox.files_list_folder(path, include_media_info=True)
-			allEntries = result.entries
-			while result.has_more:
-				result = self.dropbox.files_list_folder_continue(result.cursor)
-				allEntries += result.entries
+			try:
+				result = self.dropbox.files_list_folder(path, include_media_info=True)
+				allEntries = result.entries
+				while result.has_more:
+					result = self.dropbox.files_list_folder_continue(result.cursor)
+					allEntries += result.entries
+					if page is not None and len(allEntries) >= page[1]:
+						continue
+			except ApiError as e:
+				assert isinstance(e.error, (ListFolderError, ListFolderContinueError))
+				raise FSError() from e
 			if page is not None:
 				allEntries = allEntries[page[0]: page[1]]
 			return (_infoFromMetadata(x) for x in allEntries)
